@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gccv/text.cc 
  *
- * Copyright (C) 2008-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2008-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,6 +23,7 @@
  */
 
 #include "config.h"
+#include "canvas.h"
 #include "group.h"
 #include "text.h"
 #include "text-client.h"
@@ -105,29 +106,7 @@ void TextPrivate::OnCommit (G_GNUC_UNUSED GtkIMContext *context, const gchar *st
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-//  gccv::TextLine class implementation
-
-class TextLine
-{
-public:
-	TextLine ();
-	~TextLine ();
-
-	double m_Width, m_Height, m_BaseLine;
-	std::list <TextRun *> m_Runs;
-};
-
-TextLine::TextLine ()
-{
-	m_Width = m_Height = m_BaseLine = 0.;
-}
-
-TextLine::~TextLine ()
-{
-}
-
-////////////////////////////////////////////////////////////////////////////////
-//  gccv::TextRun class implementation
+//  gccv::TextRun class declaration
 
 class TextRun
 {
@@ -138,16 +117,161 @@ public:
 	void Draw (cairo_t *cr);
 
 	PangoLayout *m_Layout;
-	double m_X, m_Y, m_Width, m_Height, m_BaseLine;
-	unsigned m_Index, m_Length;
+	double m_X, m_Y, m_Width, m_Height, m_BaseLine, m_CharOffset;
+	unsigned m_Index, m_Length, m_NbGlyphs;
 	bool m_Stacked, m_NewLine;
 };
 
+////////////////////////////////////////////////////////////////////////////////
+//  gccv::TextLine class implementation
+
+class TextLine
+{
+public:
+	TextLine ();
+	~TextLine ();
+
+	void DrawDecorations (cairo_t *cr, bool is_vector);
+
+	double m_Width, m_Height, m_BaseLine;
+	double m_Y;
+	std::list <TextRun *> m_Runs;
+	std::list <TextTag *> m_Decorations;
+	unsigned m_Index, m_End;
+};
+
+TextLine::TextLine ()
+{
+	m_Width = m_Height = m_BaseLine = m_Y = 0.;
+}
+
+TextLine::~TextLine ()
+{
+}
+
+void TextLine::DrawDecorations (cairo_t *cr, bool is_vector)
+{
+	unsigned start, end;
+	double xstart, xend, y = 0.;
+	std::list <TextTag *>::iterator tag, end_tag = m_Decorations.end ();
+	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
+	Tag type;
+	TextDecoration dec;
+	GOColor color = 0xff;
+	for (tag = m_Decorations.begin (); tag != end_tag; tag++) {
+		type = (*tag)->GetTag ();
+		switch (type) {
+		case Underline:
+			dec = static_cast <UnderlineTextTag *> (*tag)->GetUnderline ();
+			color = static_cast <UnderlineTextTag *> (*tag)->GetColor ();
+			y = m_Y + m_BaseLine + (m_Height - m_BaseLine) * 3. / 4.;
+			if (dec == TextDecorationDefault)
+					dec = TextDecorationHigh;
+			break;
+		case Overline:
+			dec = static_cast <OverlineTextTag *> (*tag)->GetOverline ();
+			color = static_cast <OverlineTextTag *> (*tag)->GetColor ();
+			y = m_Y + (m_Height - m_BaseLine) / 4.;
+			if (dec == TextDecorationDefault)
+					dec = TextDecorationLow;
+			break;
+		case Strikethrough:
+			dec = static_cast <StrikethroughTextTag *> (*tag)->GetStrikethrough ();
+			color = static_cast <StrikethroughTextTag *> (*tag)->GetColor ();
+			y = m_Y + m_BaseLine * 3. / 4.;
+			if (dec == TextDecorationDefault)
+					dec = TextDecorationMedium;
+			break;
+		default:
+			dec = TextDecorationNone;
+			break;
+		}
+		if (dec == TextDecorationNone || color == 0)
+			continue;
+		double width = (m_Height - m_BaseLine) / 4.;
+		// find the limits as indexes
+		start = MAX (m_Index, (*tag)->GetStartIndex ());
+		end = MIN (m_End, (*tag)->GetEndIndex ());
+		// now convert to x coordinates
+		xstart = xend = 0.;
+		for (run = m_Runs.begin (); run != end_run; run++) {
+			if (start > (*run)->m_Index + (*run)->m_Length)
+				continue;
+			if (start < (*run)->m_Index)
+				xstart = (*run)->m_X;
+			else {
+				PangoRectangle rect;
+				pango_layout_get_cursor_pos ((*run)->m_Layout, start - (*run)->m_Index, &rect, NULL);
+				xstart = static_cast <double> (rect.x) / PANGO_SCALE;
+			}
+			break;
+		}
+		for (; run != end_run; run++) {
+			if (end > (*run)->m_Index + (*run)->m_Length)
+				continue;
+			if (end < (*run)->m_Index)
+				xend = (*run)->m_X;
+			else {
+				PangoRectangle rect;
+				pango_layout_get_cursor_pos ((*run)->m_Layout, end - (*run)->m_Index, &rect, NULL);
+				xend = static_cast <double> (rect.x) / PANGO_SCALE;
+			}
+		}
+		if (xstart > xend) {
+			double buf = xstart;
+			xstart = xend;
+			xend = buf;
+		}
+		if (!is_vector) {
+			double scalex = 1., scale = 1.;
+			cairo_user_to_device_distance (cr, &scalex, &scale);
+			width = round (width * scale) / scale;
+			y = round ((y + width / 2.) * scale) / scale - width / 2.;
+			xstart = round (xstart * scale) / scale;
+			xend = round (xend * scale) / scale;
+		}
+		switch (dec) {
+		case TextDecorationHigh:
+			y -= width;
+			cairo_move_to (cr, xstart, y);
+			cairo_line_to (cr, xend, y);
+			break;	
+		case TextDecorationLow:
+			y += width;
+			cairo_move_to (cr, xstart, y);
+			cairo_line_to (cr, xend, y);
+			break;	
+		case TextDecorationMedium:
+			cairo_move_to (cr, xstart, y);
+			cairo_line_to (cr, xend, y);
+			break;	
+		case TextDecorationDouble:
+			y += width;
+			cairo_move_to (cr, xstart, y);
+			cairo_line_to (cr, xend, y);
+			y -= 2 * width;
+			cairo_move_to (cr, xstart, y);
+			cairo_line_to (cr, xend, y);
+			break;	
+		default:
+			break;	
+		}
+		cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (color));
+		cairo_set_line_width (cr, width);
+		cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
+		cairo_stroke (cr);
+	}
+}
+
+////////////////////////////////////////////////////////////////////////////////
+//  gccv::TextRun class implementation
+
 TextRun::TextRun ()
 {
 	m_Layout = pango_layout_new (const_cast <PangoContext *> (Ctx.GetContext ()));
 	m_X = m_Y = 0.;
 	m_Index = m_Length = 0;
+	m_CharOffset = 0;
 	m_Stacked = m_NewLine = false;
 }
 
@@ -188,9 +312,11 @@ void TextRun::Draw (cairo_t *cr)
 	PangoRectangle rect;
 	cairo_set_source_rgba (cr, 0., 0., 0., 1.);
 	struct draw_attrs data;
+	double offset = 0; // used for justification
 	while (*text) {
 		pango_layout_iter_get_char_extents (iter, &rect);
-		curx = (double) rect.x / PANGO_SCALE;
+		curx = (double) rect.x / PANGO_SCALE + offset;
+		offset += m_CharOffset;
 		next = g_utf8_find_next_char (text, NULL);
 		data.attrs = pango_attr_list_new ();
 		data.index = index;
@@ -209,6 +335,7 @@ void TextRun::Draw (cairo_t *cr)
 		cairo_translate (cr, curx, ascent - (double) pango_layout_iter_get_baseline (local) / PANGO_SCALE);
 		pango_cairo_show_layout (cr, pl);
 		cairo_restore (cr);
+		pango_layout_iter_free (local);
 		pango_layout_iter_next_char (iter);
 	}
 	// free the iterator
@@ -224,14 +351,21 @@ Text::Text (Canvas *canvas, double x, double y):
 	m_BlinkSignal (0),
 	m_CursorVisible (false),
 	m_CurPos (0),
+	m_StartSel (0),
 	m_CurTags (NULL),
 	m_Padding (0.),
 	m_Anchor (AnchorLine),
-	m_LineOffset (0.), m_Width (0.), m_Height (0.)
+	m_LineOffset (0.),
+	m_Justification (GTK_JUSTIFY_LEFT),
+	m_Interline (0.),
+	m_Width (0.), m_Height (0.)
 {
 	TextRun *run = new TextRun ();
 	m_Runs.push_front (run);
 	m_FontDesc = pango_font_description_copy (pango_layout_get_font_description (run->m_Layout));
+	m_Lines = NULL;
+	m_LinesNumber = 0;
+	m_Color = GO_COLOR_BLACK;
 	m_ImContext = gtk_im_multicontext_new ();
 	g_signal_connect (G_OBJECT (m_ImContext), "commit",
 		G_CALLBACK (TextPrivate::OnCommit), this);
@@ -243,11 +377,13 @@ Text::Text (Group *parent, double x, double y, ItemClient *client):
 	m_BlinkSignal (0),
 	m_CursorVisible (false),
 	m_CurPos (0),
+	m_StartSel (0),
 	m_CurTags (new TextTagList ()),
 	m_Padding (0.),
 	m_Anchor (AnchorLine),
 	m_LineOffset (0.),
 	m_Justification (GTK_JUSTIFY_LEFT),
+	m_Interline (0.),
 	m_Width (0.), m_Height (0.)
 {
 	TextRun *run = new TextRun ();
@@ -255,6 +391,7 @@ Text::Text (Group *parent, double x, double y, ItemClient *client):
 	m_FontDesc = pango_font_description_copy (pango_layout_get_font_description (run->m_Layout));
 	m_Lines = NULL;
 	m_LinesNumber = 0;
+	m_Color = 0;
 	m_ImContext = gtk_im_multicontext_new ();
 	g_signal_connect (G_OBJECT (m_ImContext), "commit",
 		G_CALLBACK (TextPrivate::OnCommit), this);
@@ -312,7 +449,7 @@ void Text::SetPosition (double x, double y)
 	w = m_Width + 2 * m_Padding;
 	h = m_Height + 2 * m_Padding;
 	PangoLayoutIter* iter = pango_layout_get_iter (m_Runs.front ()->m_Layout);
-	m_Ascent = (double) pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+	m_Ascent = (double) pango_layout_iter_get_baseline (iter) / PANGO_SCALE + m_Runs.front ()->m_Y;
 	pango_layout_iter_free (iter);
 	// Horizontal position
 	switch (m_Anchor) {
@@ -426,42 +563,58 @@ void Text::Draw (cairo_t *cr, bool is_vector) const
 	}
 	// now, draw the glyphs in each run
 	std::list <TextRun *>::const_iterator i, end = m_Runs.end ();
-	for (i = m_Runs.begin (); i != end; i++) {
-		// display the selection rectangle if needed
-		if (m_BlinkSignal) {
+	if (m_BlinkSignal && start_sel != end_sel)
+		for (i = m_Runs.begin (); i != end; i++) {
+			// display the selection rectangle if needed (before drawing any text)
 			if (start_sel < (*i)->m_Index + (*i)->m_Length && end_sel > (*i)->m_Index) {
 				PangoRectangle rect;
 				unsigned s = MAX (start_sel, (*i)->m_Index) - (*i)->m_Index,
 						e = MIN (end_sel, (*i)->m_Index + (*i)->m_Length) - (*i)->m_Index;
 				double x, y, w, h;
-				cairo_set_source_rgb (cr, 0.9, 0.9, 0.9);
+				cairo_set_source_rgb (cr, 0.85, 0.85, 0.85);
 				pango_layout_get_cursor_pos ((*i)->m_Layout, s, &rect, NULL);
-				x = (double) rect.x / PANGO_SCALE;
+				x = (double) rect.x / PANGO_SCALE + (s > 0? s - 1: 0) * (*i)->m_CharOffset;
 				h = (double) rect.height / PANGO_SCALE;
 				y = starty + (*i)->m_Y + (double) rect.y / PANGO_SCALE;
 				pango_layout_get_cursor_pos ((*i)->m_Layout, e, &rect, NULL);
-				w = (double) rect.x / PANGO_SCALE - x;
+				w = (double) rect.x / PANGO_SCALE + (e > 0? e - 1: 0) * (*i)->m_CharOffset - x;
 				x += startx + (*i)->m_X;
 				cairo_rectangle (cr, x, y, w, h);
 				cairo_fill (cr);
 			}
 		}
+	// now, draw the glyphs in each run
+	for (i = m_Runs.begin (); i != end; i++) {
 		// draw the text
 		cairo_save (cr);
 		cairo_translate (cr, startx + (*i)->m_X, starty + (*i)->m_Y);
 		(*i)->Draw (cr);
 		cairo_restore (cr);
 		// draw the cursor if needed
-		if (m_CursorVisible && m_CurPos > (*i)->m_Index && m_CurPos <= (*i)->m_Index + (*i)->m_Length) {
+		if (m_CursorVisible && ((m_CurPos > (*i)->m_Index && m_CurPos <= (*i)->m_Index + (*i)->m_Length) ||
+		    ((*i)->m_NewLine && m_CurPos == (*i)->m_Index))) {
 			PangoRectangle rect;
 			pango_layout_get_cursor_pos ((*i)->m_Layout, m_CurPos - (*i)->m_Index, &rect, NULL);
 			cairo_set_line_width (cr, 1.);
-			cairo_move_to (cr, floor (startx + (*i)->m_X + (double) rect.x / PANGO_SCALE) + .5, floor (starty + (*i)->m_Y + (double) rect.y / PANGO_SCALE) + .5);
+			int nbskip = (m_CurPos > (*i)->m_Index)? m_CurPos - (*i)->m_Index - 1: 0;
+			cairo_move_to (cr, floor (startx + (*i)->m_X + (double) rect.x / PANGO_SCALE + nbskip * (*i)->m_CharOffset) + .5, floor (starty + (*i)->m_Y + (double) rect.y / PANGO_SCALE) + .5);
 			cairo_rel_line_to (cr, 0, rect.height / PANGO_SCALE);
 			cairo_set_source_rgb (cr, 0., 0., 0.);
 			cairo_stroke (cr);
 		}
 	}
+	// draw decorations (underline and friends)
+	cairo_save (cr);
+	if (!is_vector) {
+		double scalex = 1., scale = 1.;
+		cairo_user_to_device_distance (cr, &scalex, &scale);
+		startx = round (startx * scale) / scale;
+		starty = round (starty * scale) / scale;
+	}
+	cairo_translate (cr, startx, starty);
+	for (unsigned line = 0; line < m_LinesNumber; line++)
+		m_Lines[line].DrawDecorations (cr, is_vector);
+	cairo_restore (cr);
 	if (m_CursorVisible && m_CurPos == 0) {
 		PangoRectangle rect;
 		pango_layout_get_cursor_pos (m_Runs.front ()->m_Layout, m_CurPos, &rect, NULL); // FIXME: might be wrong if we allow above-under characters
@@ -487,24 +640,27 @@ PangoContext *Text::GetContext ()
 void Text::SetText (char const *text)
 {
 	m_Text = text;
-	pango_layout_set_text (m_Runs.front ()->m_Layout, text, -1); // FIXME: parse for line breaks
-	m_Runs.front ()->m_Length = strlen (text);
+	// clear all attributes
+	while (!m_Tags.empty ()) {
+		delete m_Tags.front ();
+		m_Tags.pop_front ();
+	}
+	RebuildAttributes ();
 }
 
 void Text::SetText (std::string const &text)
 {
 	m_Text = text;
-	pango_layout_set_text (m_Runs.front ()->m_Layout, text.c_str (), -1); // FIXME: parse for line breaks
-	m_Runs.front ()->m_Length = strlen (text.c_str ());
-	SetPosition (m_x, m_y);
+	// clear all attributes
+	while (!m_Tags.empty ()) {
+		delete m_Tags.front ();
+		m_Tags.pop_front ();
+	}
+	RebuildAttributes ();
 }
 
 char const *Text::GetText ()
 {
-	std::list <TextRun *>::iterator i = m_Runs.begin (), end = m_Runs.end ();
-	m_Text = pango_layout_get_text ((*i)->m_Layout);
-	for (i++; i != end; i++)
-		m_Text += pango_layout_get_text ((*i)->m_Layout);
 	return m_Text.c_str ();
 }
 
@@ -640,14 +796,49 @@ void Text::GetBounds (Rect *ink, Rect *logical)
 
 void Text::InsertTextTag (TextTag *tag, bool rebuild_attributes)
 {
-	// FIXME: we need to filter tags to avoid duplicates
+	// we need to filter tags to avoid duplicates
+	std::list <TextTag *> new_tags, invalid;
+	TextTagList::iterator i, iend = m_Tags.end ();
+	TextTag *new_tag;
+	for (i = m_Tags.begin (); i != m_Tags.end (); i++) {
+		new_tag = tag->Restrict (*i);
+		if (new_tag)
+			new_tags.push_back (new_tag);
+		if ((*i)->GetStartIndex () >= (*i)->GetEndIndex ())
+			invalid.push_back (*i);
+	}
+	// no remove invalid tags
+	while (!invalid.empty ()) {
+		delete invalid.front ();
+		m_Tags.remove (invalid.front ());
+		invalid.pop_front ();
+	}
+	// add split tags
+	while (!new_tags.empty ()) {
+		new_tag = new_tags.front ();
+		new_tags.pop_front ();
+		if (new_tag->GetPriority () == TagPriorityFirst)
+			m_Tags.push_front (new_tag);
+		else
+			m_Tags.push_back (new_tag);
+	}
 	// now, insert the new tag
 	if (tag->GetPriority () == TagPriorityFirst)
 		m_Tags.push_front (tag);
 	else
 		m_Tags.push_back (tag);
 	// now, rebuild pango attributes lists for modified runs.
-	// FIXME: it would be better to update only the changed portion
+	// Note: it would be better to update only the changed portion, but how???
+	if (rebuild_attributes)
+		RebuildAttributes ();
+}
+
+void Text::DeleteTextTag (TextTag *tag, bool rebuild_attributes)
+{
+	if (!tag)
+		return;
+	m_Tags.remove (tag);
+	delete tag;
 	if (rebuild_attributes)
 		RebuildAttributes ();
 }
@@ -825,10 +1016,9 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 	if (nl == 0) {
 		pango_layout_set_text (m_Runs.front ()->m_Layout, m_Text.c_str (), -1); // FIXME: parse for line breaks and update runs
 		m_Runs.front ()->m_Length = m_Text.length ();
-		RebuildAttributes ();
 		m_CurPos = m_StartSel = pos;
+		RebuildAttributes ();
 		SetPosition (m_x, m_y);
-		//TODO: update attribute list
 		return;
 	}
 	m_Text.insert (pos, str);
@@ -838,7 +1028,7 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 	for (i = m_Tags.begin (); i != iend; i++) {
 		if ((*i)->GetEndIndex () < static_cast <unsigned> (pos))
 			continue;
-		if ((*i)->GetStartIndex () > static_cast <unsigned> (pos)) {
+		if ((*i)->GetStartIndex () >= static_cast <unsigned> (pos)) {
 			(*i)->SetStartIndex ((*i)->GetStartIndex () + nl);
 			(*i)->SetEndIndex ((*i)->GetEndIndex () + nl);
 		}
@@ -884,9 +1074,8 @@ void Text::ReplaceText (std::string &str, int pos, unsigned length)
 	}
 	extra_tags.clear (); // avoid destroying the current tags
 	pango_layout_set_text (m_Runs.front ()->m_Layout, m_Text.c_str (), -1); // FIXME: parse for line breaks and update runs
-	m_Runs.front ()->m_Length = m_Text.length ();
-	RebuildAttributes ();
 	m_CurPos = m_StartSel = pos + str.length ();
+	RebuildAttributes ();
 	SetPosition (m_x, m_y);
 }
 
@@ -906,10 +1095,19 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 	case GDK_Control_R:
 		return false;
 	case GDK_Return:
-	case GDK_KP_Enter:
-		/* TODO: write this code */
-		break;
-
+	case GDK_KP_Enter: {
+		m_Text.insert (m_CurPos, "\n");
+		TextTag *tag = new NewLineTextTag ();
+		tag->SetStartIndex (m_CurPos++);
+		tag->SetEndIndex (m_CurPos);
+		m_StartSel = m_CurPos;
+		m_Tags.push_front (tag);
+		RebuildAttributes ();
+		SetPosition (m_x, m_y);
+		if (client)
+			client->TextChanged (m_CurPos);
+		return true;
+	}
 	case GDK_Tab:
 		TextPrivate::OnCommit (m_ImContext, "\t", this);
 		if (client)
@@ -1069,6 +1267,17 @@ bool Text::OnKeyPressed (GdkEventKey *event)
 	return true;
 }
 
+bool Text::OnKeyReleased (GdkEventKey *event)
+{
+	TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+	if (gtk_im_context_filter_keypress (m_ImContext, event)) {
+		if (client)
+			client->TextChanged (m_CurPos);
+		return true;
+	}
+	return false;
+}
+
 void Text::RebuildAttributes ()
 {
 	// This might be optimized: not everything should be scanned each time a change occur
@@ -1079,7 +1288,7 @@ void Text::RebuildAttributes ()
 	for (run = m_Runs.begin ();run != end_run; run++) // delete all runs
 		delete (*run);
 	m_Runs.clear ();
-	// Recreate first run
+	// Recreate first run and first line
 	TextRun *new_run = new TextRun (), *last_run;
 	pango_layout_set_font_description (new_run->m_Layout, m_FontDesc);
 	m_Runs.push_front (new_run);
@@ -1088,13 +1297,27 @@ void Text::RebuildAttributes ()
 	bool stacked = false;
 	unsigned lines = 1;
 	std::string str;
+	TextTagList decorations;
+	unsigned cur_end = 0;
 	for (tag = m_Tags.begin (); tag != end_tag; tag++) {
 		if (stacked || (*tag)->GetStacked ()) {
 			// we need a new run
+			if (cur_end < (*tag)->GetStartIndex () && last_run->m_Stacked) {
+				new_run = new TextRun ();
+				pango_layout_set_font_description (new_run->m_Layout, m_FontDesc);
+				new_run ->m_Index = cur_end;
+				last_run->m_Length = new_run->m_Index - last_run->m_Index;
+				last_run->m_NbGlyphs = g_utf8_strlen (m_Text.c_str () + last_run->m_Index, last_run->m_Length);
+				new_run->m_Stacked = false;
+				m_Runs.push_back (new_run);
+				last_run = new_run;
+			}
+			cur_end = (*tag)->GetEndIndex ();
 			new_run = new TextRun ();
 			pango_layout_set_font_description (new_run->m_Layout, m_FontDesc);
 			new_run ->m_Index = (*tag)->GetStartIndex ();
 			last_run->m_Length = new_run->m_Index - last_run->m_Index;
+			last_run->m_NbGlyphs = g_utf8_strlen (m_Text.c_str () + last_run->m_Index, last_run->m_Length);
 			stacked = new_run->m_Stacked = (*tag)->GetStacked ();
 			m_Runs.push_back (new_run);
 			last_run = new_run;
@@ -1106,12 +1329,16 @@ void Text::RebuildAttributes ()
 			pango_layout_set_font_description (new_run->m_Layout, m_FontDesc);
 			new_run ->m_Index = (*tag)->GetStartIndex ();
 			last_run->m_Length = new_run->m_Index - last_run->m_Index;
+			last_run->m_NbGlyphs = g_utf8_strlen (m_Text.c_str () + last_run->m_Index, last_run->m_Length);
+			new_run->m_Index++; // skip the new line character
 			new_run->m_NewLine = true;
 			m_Runs.push_back (new_run);
 			last_run = new_run;
-		}
+		} else if ((*tag)->GetTag () == Underline || (*tag)->GetTag () == Strikethrough || (*tag)->GetTag () == Overline)
+			decorations.push_back (*tag);
 	}
 	last_run->m_Length = m_Text.length () - last_run->m_Index;
+	last_run->m_NbGlyphs = g_utf8_strlen (m_Text.c_str () + last_run->m_Index, last_run->m_Length);
 	// now update attributes for each run
 	end_run = m_Runs.end ();
 	PangoLayoutIter *iter;
@@ -1119,6 +1346,13 @@ void Text::RebuildAttributes ()
 		str = m_Text.substr ((*run)->m_Index, (*run)->m_Length);
 		pango_layout_set_text ((*run)->m_Layout, str.c_str (), -1);
 		PangoAttrList *l = pango_attr_list_new ();
+		// set the default text color
+		if (m_Color) {
+			PangoAttribute *attr = pango_attr_foreground_new (GO_COLOR_UINT_R (m_Color) * 0x101, GO_COLOR_UINT_G (m_Color) * 0x101, GO_COLOR_UINT_B (m_Color) * 0x101);
+			attr->start_index = 0;
+			attr->end_index = (*run)->m_Length;
+			pango_attr_list_insert (l, attr);
+		}
 		for (tag = m_Tags.begin (); tag != end_tag; tag++) {
 			if ((*tag)->GetEndIndex () <= (*run)->m_Index || (*tag)->GetStartIndex () >= (*run)->m_Index + (*run)->m_Length)
 				continue;
@@ -1142,31 +1376,117 @@ void Text::RebuildAttributes ()
 	m_LinesNumber = lines;
 	// we now need to rebuild runs positions
 	// FIXME: support several lines
-	double curx = 0., curw = 0.;
+	double curx = 0., curw = 0., cury = 0;
 	unsigned cur_line = 0;
+	m_Lines[0].m_Index = 0;
+	double width = 0.;
 	for (run = m_Runs.begin (); run != end_run; run++) {
-		m_Lines[cur_line].m_Runs.push_back (*run);
 		if ((*run)->m_Stacked) {
 			if ((*run)->m_Width > curw)
 				curw = (*run)->m_Width;
 			(*run)->m_X = curx;
 		} else if ((*run)->m_NewLine) {
-			m_Lines[cur_line].m_Width = curx + curw;
+			curw += curx;
+			m_Lines[cur_line].m_Width = curw;
+			m_Lines[cur_line].m_End = (*run)->m_Index;
+			if (width < curw)
+				width = curw;
+			curw = 0.;
+			curx = (*run)->m_Width;
 			cur_line++;
+			m_Lines[cur_line].m_Index = (*run)->m_Index - 1; // -1 because of the hidden \n
 		} else {
 			(*run)->m_X = curx + curw;
-			curx += (*run)->m_Width;
+			curx += curw + (*run)->m_Width;
 			curw = 0.;
 		}
+		m_Lines[cur_line].m_Runs.push_back (*run);
 		if (m_Lines[cur_line].m_BaseLine < (*run)->m_BaseLine)
 			m_Lines[cur_line].m_BaseLine = (*run)->m_BaseLine;
+		if (m_Lines[cur_line].m_Height < (*run)->m_Height)
+			m_Lines[cur_line].m_Height = (*run)->m_Height;
 	}
-	m_Lines[cur_line].m_Width = curx + curw;
+	curw += curx;
+	m_Lines[cur_line].m_Width = curw;
+	run --;
+	m_Lines[cur_line].m_End = (*run)->m_Index + (*run)->m_Length;
+	if (width < curw)
+		width = curw;
+	end_tag = decorations.end ();
 	for (cur_line = 0; cur_line < lines; cur_line++) {
+		m_Lines[cur_line].m_Y = cury;
 		end_run = m_Lines[cur_line].m_Runs.end ();
 		for (run = m_Lines[cur_line].m_Runs.begin (); run != end_run; run++)
-			(*run)->m_Y = m_Lines[cur_line].m_BaseLine - (*run)->m_BaseLine;
+			(*run)->m_Y = cury + m_Lines[cur_line].m_BaseLine - (*run)->m_BaseLine;
+		cury += m_Lines[cur_line].m_Height + m_Interline;
+		// now manage justification
+		if (m_Justification != GTK_JUSTIFY_LEFT && m_Lines[cur_line].m_Width < width) {
+			switch (m_Justification) {
+			case GTK_JUSTIFY_RIGHT:
+				for (run = m_Lines[cur_line].m_Runs.begin (); run != end_run; run++)
+					(*run)->m_X += width - m_Lines[cur_line].m_Width;
+				break;
+			case GTK_JUSTIFY_CENTER:
+				for (run = m_Lines[cur_line].m_Runs.begin (); run != end_run; run++)
+					(*run)->m_X += (width - m_Lines[cur_line].m_Width) / 2.;
+				break;
+			case GTK_JUSTIFY_FILL: {
+				// this is a bit more difficult, current code might be suboptimal if there are stacked runs
+				// evaluate how many characters are in the line
+				unsigned nbchars = 0;
+				stacked = false;
+				unsigned *nc = new unsigned[m_Lines[cur_line].m_Runs.size ()]; // this is more than needed if we have stacked runs
+				int cur, max = 0;
+				nc[0] = 0;	// fortunately, we have at least one run
+				for (run = m_Lines[cur_line].m_Runs.begin (); run != end_run; run++) {
+					if ((*run)->m_Stacked) {
+						stacked = true;
+						if (nc[max] < (*run)->m_NbGlyphs)
+							nc[max] = (*run)->m_NbGlyphs;
+					} else {
+						if (stacked)
+							max++;
+						nc[max++] = (*run)->m_NbGlyphs;
+						nc[max] = 0;
+						stacked = false;
+					}
+				}
+				if (stacked)
+					max++;
+				for (cur = 0; cur < max; cur++)
+					nbchars += nc[cur];
+				// calculate the increment to add between two glyphs
+				double incr = (width - m_Lines[cur_line].m_Width) / (nbchars - 1), curoffs = 0.;
+				// update the runs accordingly
+				cur = 0;
+				for (run = m_Lines[cur_line].m_Runs.begin (); run != end_run; run++) {
+					(*run)->m_X += curoffs;
+					(*run)->m_CharOffset = incr;
+					if ((*run)->m_Stacked)
+						stacked = true;
+					else {
+						if (stacked) {
+							stacked = false;
+							curoffs += nc[cur] * incr;
+							cur++;
+						}
+						curoffs += nc[cur] * incr;
+						cur++;
+					}
+				}
+				delete [] nc;
+				break;
+			}
+			default:
+				break;
+			}
+		}
+		// now find which decorations apply to the line
+		for (tag = decorations.begin (); tag != end_tag; tag++)
+			if ((*tag)->GetStartIndex () < m_Lines[cur_line].m_End && (*tag)->GetEndIndex () > m_Lines[cur_line].m_Index)
+				m_Lines[cur_line].m_Decorations.push_back (*tag);
 	}
+	decorations.clear ();
 	// force reposition and redraw
 	SetPosition (m_x, m_y);
 }
@@ -1177,33 +1497,31 @@ void Text::OnButtonPressed (double x, double y)
 	GetParent ()->AdjustBounds (x0, y0, x1, y1);
 	x -= x0;
 	y -= y0;
-	int index, trailing;
-	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
-	for (run = m_Runs.begin (); run != end_run; run++)
-		if (pango_layout_xy_to_index ((*run)->m_Layout, x * PANGO_SCALE, y * PANGO_SCALE, &index, &trailing)) {
-			m_CurPos = m_StartSel = index + trailing + (*run)->m_Index;
-			TextClient *client = dynamic_cast <TextClient *> (GetClient ());
-			if (client)
-				client->SelectionChanged (m_StartSel, m_CurPos);
-			return;
-		}
+	unsigned index = GetIndexAt (x, y);
+	if (index < G_MAXUINT) {
+		m_CurPos = index;
+		if ((GetCanvas ()->GetLastEventState () & GDK_SHIFT_MASK) == 0)
+			m_StartSel = m_CurPos;
+		TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+		if (client)
+			client->SelectionChanged (m_StartSel, m_CurPos);
+	}
 }
 
 void Text::OnDrag (double x, double y)
 {
-	int index, trailing;
-	x -= m_x0;
-	y -= m_y0;
-	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
-	for (run = m_Runs.begin (); run != end_run; run++)
-		if (pango_layout_xy_to_index ((*run)->m_Layout, x * PANGO_SCALE, y * PANGO_SCALE, &index, &trailing)) {
-			m_CurPos = index + trailing + (*run)->m_Index;
-			Invalidate ();
-			TextClient *client = dynamic_cast <TextClient *> (GetClient ());
-			if (client)
-				client->SelectionChanged (m_StartSel, m_CurPos);
-			return;
-		}
+	double x0 = m_x0, y0 = m_y0, x1 = 0., y1 = 0.;
+	GetParent ()->AdjustBounds (x0, y0, x1, y1);
+	x -= x0;
+	y -= y0;
+	unsigned index = GetIndexAt (x, y);
+	if (index < G_MAXUINT) {
+		m_CurPos = index;
+		Invalidate ();
+		TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+		if (client)
+			client->SelectionChanged (m_StartSel, m_CurPos);
+	}
 }
 
 void Text::GetSelectionBounds (unsigned &start, unsigned &end)
@@ -1225,12 +1543,74 @@ void Text::SetSelectionBounds (unsigned start, unsigned end)
 
 unsigned Text::GetIndexAt (double x, double y)
 {
-	int index, trailing;
-	std::list <TextRun *>::iterator run, end_run = m_Runs.end ();
-	for (run = m_Runs.begin (); run != end_run; run++)
-		if (pango_layout_xy_to_index ((*run)->m_Layout, x * PANGO_SCALE, y * PANGO_SCALE, &index, &trailing))
-			return index + trailing + (*run)->m_Index;
-	return G_MAXUINT;
+	// first, find the line
+	double starty = m_Interline / 2.;
+	unsigned line = 0;
+	unsigned result = 0;
+	double runx = 0.;
+	while (line < m_LinesNumber - 1) {
+		if (y < starty + m_Lines[line].m_Height)
+			break;
+		starty += m_Lines[line].m_Height + m_Interline;
+		line++;
+	}
+	// now, find the run
+	std::list <TextRun *>::iterator run, end_run = m_Lines[line].m_Runs.end (), srun;
+	for (run = m_Lines[line].m_Runs.begin (); run != end_run; run++) {
+		runx = x - (*run)->m_X;
+		if (runx <= (*run)->m_Width + (*run)->m_NbGlyphs * (*run)->m_CharOffset)
+			break;
+	}
+	if (run == end_run)
+		run--;
+	// stacked, choose the right run as far as possible
+	if ((*run)->m_Stacked) {
+		while (run != m_Lines[line].m_Runs.begin () && (*run)->m_Stacked)
+			run--;
+		if (!(*run)->m_Stacked)
+			run++;
+		// we are now at the first run in the stack
+		PangoRectangle rect;
+		double ymin, ymax, dy = G_MAXDOUBLE;
+		while (run != end_run && (*run)->m_Stacked) {
+			pango_layout_get_extents ((*run)->m_Layout, &rect, NULL);
+			ymin = (*run)->m_Y + static_cast <double> (rect.y) / PANGO_SCALE;
+			ymax = ymin + static_cast <double> (rect.height) / PANGO_SCALE;
+			if (y < ymin) {
+				if (ymin - y < dy) {
+					dy = ymin - y;
+					srun = run;
+				}
+			} else if (y < ymax) {
+				srun = run;
+				break;
+			} else {
+				if (y - ymax < dy) {
+					dy = y - ymax;
+					srun = run;
+				}
+			}
+			run++;
+		}
+		run = srun;
+	}
+	PangoLayoutIter* iter = pango_layout_get_iter ((*run)->m_Layout);
+	PangoRectangle rect;
+	pango_layout_iter_get_char_extents (iter, &rect);
+	double curx = 0.;
+	while (curx < runx) {
+		if ((double) rect.width / PANGO_SCALE / 2. >= x - curx)
+			break;
+		result++;
+		curx += (double) rect.width / PANGO_SCALE + (*run)->m_CharOffset;
+		if (!pango_layout_iter_next_char (iter))
+			break;
+		pango_layout_iter_get_char_extents (iter, &rect);
+	}
+	pango_layout_iter_free (iter);
+	if ((result += (*run)->m_Index) > m_Text.length ())
+		result = m_Text.length ();
+	return result;
 }
 
 bool Text::GetPositionAtIndex (unsigned index, Rect &rect)
@@ -1242,13 +1622,43 @@ bool Text::GetPositionAtIndex (unsigned index, Rect &rect)
 		if (index <= (*run)->m_Index + (*run)->m_Length) {
 			PangoRectangle r;
 			pango_layout_index_to_pos ((*run)->m_Layout, index - (*run)->m_Index, &r);
-			rect.x0 = (double) r.x / PANGO_SCALE;
-			rect.y0 = (double) r.y / PANGO_SCALE;
-			rect.x1 = (double) (r.x + r.width) / PANGO_SCALE;
-			rect.y1 = (double) (r.y + r.height) / PANGO_SCALE;
+			rect.x0 = (double) r.x / PANGO_SCALE + (*run)->m_X + (index - (*run)->m_Index) * (*run)->m_CharOffset;
+			rect.y0 = (double) r.y / PANGO_SCALE + (*run)->m_Y;
+			rect.x1 = rect.x0 + (double) r.width / PANGO_SCALE;
+			rect.y1 = (double) (r.y + r.height) / PANGO_SCALE + (*run)->m_Y;
 			break;
 		}
 	return true;
 }
 
+void Text::SetColor (GOColor color)
+{
+	m_Color = color;
+	RebuildAttributes ();
+}
+
+void Text::SetInterline (double interline, bool emit_changed)
+{
+	m_Interline = interline;
+	RebuildAttributes ();
+	SetPosition (m_x, m_y);
+	if (emit_changed) {
+		TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+		if (client)
+			client->InterlineChanged (interline);
+	}
+}
+
+void Text::SetJustification (GtkJustification justification, bool emit_changed)
+{
+	m_Justification = justification;
+	RebuildAttributes ();
+	Invalidate ();
+	if (emit_changed) {
+		TextClient *client = dynamic_cast <TextClient *> (GetClient ());
+		if (client)
+			client->JustificationChanged (justification);
+	}
+}
+
 }
