@@ -4,7 +4,7 @@
  * GChemPaint library
  * mechanism-step.cc 
  *
- * Copyright (C) 2009-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2009-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,9 +23,11 @@
  */
 
 #include "config.h"
+#include "document.h"
 #include "mechanism-step.h"
 #include "mechanism-arrow.h"
 #include "molecule.h"
+#include "operation.h"
 #include "reaction-step.h"
 #include <glib/gi18n.h>
 
@@ -80,9 +82,10 @@ bool MechanismStep::OnSignal (gcu::SignalId Signal, G_GNUC_UNUSED gcu::Object *C
 				if (mol)
 					molecules.insert (mol);
 			}
+		Object *parent = GetParent (), *group = GetGroup ();
+		Operation *op = static_cast <Document *> (GetDocument ())->GetCurrentOperation ();
 		if (molecules.empty ()) {
 			// if no arrow remains, delete this
-			Object *parent = GetParent ();
 			Object *obj;
 			SetParent (NULL); //
 			if (parent->GetType () == ReactionStepType) {
@@ -91,34 +94,45 @@ bool MechanismStep::OnSignal (gcu::SignalId Signal, G_GNUC_UNUSED gcu::Object *C
 					if (obj->GetType () == gcu::MoleculeType)
 						step->AddMolecule (static_cast <Molecule *> (obj), false);
 			} else
-				while ((obj = GetFirstChild (i)))
+				while ((obj = GetFirstChild (i))) {
 					parent->AddChild (obj);
+					if (!group && op)
+						op->AddObject (obj, 1);
+						
+				}
 			delete this;
 			parent->EmitSignal (OnChangedSignal);
 			return false;
 		} else {
-			Object *parent = GetParent ();
 			Object *obj;
-			ReactionStep *step = static_cast <ReactionStep *> (parent);
-			std::set <Object *> orphans;
-			std::set <Object *>::iterator j, end = molecules.end ();
-			// search for molecules without a mechanism arrow
-			for (obj = GetFirstChild (i); obj; obj = GetNextChild (i))
-				if (molecules.find (obj) == end)
-					orphans.insert (obj);
-			// now remove orphans from this
-			j = orphans.end ();
-			for (j = orphans.begin (); j != end; j++) {
-				if (step)
-					step->AddMolecule (static_cast <Molecule *> (*j), false);
-				else
-					parent->AddChild (*j);
+			if (parent->GetType () == ReactionStepType) {
+				ReactionStep *step = static_cast <ReactionStep *> (parent);
+				std::set <Object *> orphans;
+				std::set <Object *>::iterator j, end = molecules.end ();
+				// search for molecules without a mechanism arrow
+				for (obj = GetFirstChild (i); obj; obj = GetNextChild (i))
+					if (molecules.find (obj) == end)
+						orphans.insert (obj);
+				// now remove orphans from this
+				j = orphans.end ();
+				for (j = orphans.begin (); j != end; j++) {
+					if (step)
+						step->AddMolecule (static_cast <Molecule *> (*j), false);
+					else
+						parent->AddChild (*j);
+				}
 			}
 		}
 	}
 	return true;
 }
 
+xmlNodePtr MechanismStep::Save (xmlDocPtr xml) const
+{
+	// don't save an empty step, note that this might not occur
+	return (GetChildrenNumber ())? gcu::Object::Save (xml): NULL;
+}
+
 bool MechanismStep::Load (xmlNodePtr node)
 {
 	m_bLoading = true;
