@@ -461,6 +461,7 @@ Window::Window (gcp::Application *App, char const *Theme, char const *extra_ui)
 		GError *error;
 
 	SetWindow (window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL)));
+	gtk_window_set_icon_name (window, App->GetIconName ().c_str ());
 	g_object_set (G_OBJECT (window), "urgency-hint", false, NULL);
 	g_object_set_data (G_OBJECT (window), "gcp-role", (void*) 1);
 	g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (on_destroy), this);
@@ -562,6 +563,7 @@ Window::Window (gcp::Application *App, char const *Theme, char const *extra_ui)
 
 Window::~Window ()
 {
+	g_object_unref (m_UIManager);
 }
 
 void Window::OnFileNew()
@@ -744,7 +746,7 @@ bool Window::VerifySaved ()
 	} while ((res == GTK_RESPONSE_YES) && (m_Document->GetFileName () == NULL));
 	if (res == GTK_RESPONSE_NO)
 		m_Document->SetDirty (false);
-	return (res != GTK_RESPONSE_CANCEL);
+	return (res == GTK_RESPONSE_YES || res == GTK_RESPONSE_NO);
 }
 
 void Window::OnPageSetup ()
