@@ -156,6 +156,7 @@ void CrystalDoc::ParseXMLTree (xmlNode* xml)
 	bool bViewLoaded = false;
 
 	Reinit ();
+	m_SpaceGroup = NULL;
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	//look for generator node
@@ -793,7 +794,25 @@ bool CrystalDoc::Loaded () throw (LoaderError)
 		}
 		(*i)->SetDefaultColor ();
 	}
-	switch (m_lattice) { // ensure that the paraleters are coherent
+	if (m_lattice == triclinic) { // this is for invalid files
+		if (m_alpha == m_beta) {
+			if (m_alpha == m_gamma) {
+				if (m_alpha == 90) {
+					if (m_a != m_b)
+						m_lattice = orthorhombic;
+					else if (m_a != m_c)
+						m_lattice = tetragonal;
+					else
+						m_lattice = cubic;
+				} else
+					m_lattice = rhombohedral;
+			}
+		} else if (m_alpha == 90  && m_gamma == 90)
+			m_lattice = monoclinic;
+		else if (m_alpha == 90  && m_gamma == 120)
+				m_lattice = hexagonal;
+	}
+	switch (m_lattice) { // ensure that the parameters are coherent
 	case cubic:
 	case body_centered_cubic:
 	case face_centered_cubic:
@@ -817,8 +836,15 @@ bool CrystalDoc::Loaded () throw (LoaderError)
 		m_alpha = m_beta = m_gamma  = 90;
 		break;
 	case rhombohedral:
-		m_beta = m_gamma = m_alpha;
-		m_b = m_c = m_a;
+		if (m_alpha != 90) {
+			m_beta = m_gamma = m_alpha;
+			m_b = m_c = m_a;
+		} else {
+			// looks like the usual convention, at least in CIF files.
+			m_alpha = m_beta = 90;
+			m_gamma  = 120;
+			m_b = m_a;
+		}
 		break;
 	case monoclinic:
 	case base_centered_monoclinic:
@@ -890,11 +916,12 @@ bool CrystalDoc::SetProperty (unsigned property, char const *value)
 			}
 		else if (id <= 142)
 			m_lattice = (type == 'P')? tetragonal: body_centered_tetragonal;
-		else if (id <= 167)
-			m_lattice = rhombohedral;
-		else if (id <= 194)
-			m_lattice = hexagonal;
-		else {
+		else if (id <= 194) {
+			if (id == 146 || id == 148 || id == 155 || id == 160 || id == 161 || id == 166 || id == 167)
+				m_lattice = rhombohedral;
+			else
+				m_lattice = hexagonal;
+		} else {
 			switch (type) {
 			case 'P':
 				m_lattice = cubic;
