@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/glview.h 
  *
- * Copyright (C) 2006-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2006-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,15 +28,7 @@
 #include "macros.h"
 #include "matrix.h"
 #include "printable.h"
-#ifdef HAVE_GO_CONF_SYNC
-#   ifdef GOFFICE_HAS_GLOBAL_HEADER
-#      include <goffice/goffice.h>
-#   else
-#      include <goffice/app/go-conf.h>
-#   endif
-#else
-#   include <gconf/gconf-client.h>
-#endif
+#include <goffice/goffice.h>
 #include <gtk/gtkwidget.h>
 #include <map>
 #include <string>
@@ -81,7 +73,7 @@ Initialize the associated widget. Automatically called by the framework.
 /*!
 Automatically called by the framework when the associated widget size changes.
 */
-	void Reshape ();
+	void Reshape (int width, int height);
 /*!
 Draws the contents of the associated widget. Automatically called by the framework.
 */
@@ -138,10 +130,11 @@ Generates a pixbuf from the current view.
 /*!
 @param print a GtkPrintOperation.
 @param context a GtkPrintContext.
+@param page the page to print.
 
 Prints the current view at 300 dpi.
 */
-	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const;
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context, int page) const;
 /*!
 @return the top level GtkWindow containing the view.
 */
@@ -173,13 +166,10 @@ The associated widget.
 private:
 	bool m_bInit;
 	Matrix m_Euler;
+	int m_WindowHeight, m_WindowWidth;
 	double m_Height, m_Width, m_Near, m_Far;
 	double m_Lastx, m_Lasty;
-#ifdef HAVE_GO_CONF_SYNC
 	static GOConfNode *m_ConfNode;
-#else
-	static GConfClient *m_ConfClient;
-#endif
 	static guint m_NotificationId;
 	static int nbViews;
 
