@@ -2,7 +2,7 @@
  * Gnome Chemistry Utils
  * value.h 
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,6 +29,7 @@
 #include <string>
 #include <stdexcept>
 
+/*!\file*/
 namespace gcu
 {
 
@@ -70,6 +71,11 @@ public:
 Default constructor
 */
 	SimpleValue ();
+/*!
+@param value a GcuValue to embed.
+
+Constructs a SimpleValue \a value.
+*/
 	SimpleValue (GcuValue value);
 /*!
 Default destructor
@@ -88,7 +94,19 @@ Default destructor
 @return the embedded GcuValue structure.
 */
 	GcuValue const GetValue () {return val;}
+/*!
+@param value the value to add.
+
+Adds two values.
+@return the result of the addition.
+*/
 	SimpleValue operator+ (SimpleValue const &value) const;
+/*!
+@param n the number used to multiply.
+
+Multiplies \a this by \a n.
+@return the result of the multiplication.
+*/
 	SimpleValue operator* (int n) const;
 
 private:
@@ -121,13 +139,25 @@ Default destructor
 @return the embedded value. Note that the unit is lost.
 */
 	double GetAsDouble () const;
+/*!
+@param value the value to add.
 
+Adds two values. They need to have the same dimension, otherwise an exception
+will be thrown.
+@return the result of the addition.
+*/
 	DimensionalValue operator+ (DimensionalValue const &value) const throw (std::invalid_argument);
+/*!
+@param n the number used to multiply.
+
+Multiplies \a this by \a n.
+@return the result of the multiplication.
+*/
 	DimensionalValue operator* (int n) const;
 /*!
 @return the embedded GcuDimensionalValue structure.
 */
-	GcuDimensionalValue const GetValue () {return val;}
+	GcuDimensionalValue const GetValue () const {return val;}
 
 private:
 	GcuDimensionalValue val;
